@@ -10,6 +10,9 @@ import itertools
 
 ## TODO
 # 1. Make it impossible to pick multiple vertexes at once. (right now if vertex are too close they will be selected.)
+# 2. refactor events?
+# 3. Refactor/rename FCToolBar -> FCGateManager
+# 4. Make a separate file for drawing tools
 
 def apply_format(var, format_str):
     """ Formats all non-iterables inside of the iterable var using the format_str
@@ -156,7 +159,6 @@ class BaseVertex(EventGenerator):
                 svertex.update_position(verts[0], verts[1])
         self.callback(Event(Event.BASE_GATE_CHANGED))
 
-
 class SpawnableVertex(AxesWidget, EventGenerator):
     """
     Defines a moveable vertex. The vertex must be associated
@@ -262,7 +264,7 @@ class SpawnableVertex(AxesWidget, EventGenerator):
             self.artist.set_ydata([ydata])
 
     def _update(self):
-        self.canvas.draw()
+        self.canvas.draw_idle()
 
     def update_looks(self, state):
         if state == 'active':
@@ -369,13 +371,6 @@ class BaseGate(EventGenerator):
             gate_type_name = 'QuadGate'
         return gate_type_name
 
-
-
-    def set_axis(self, ch, ax):
-        self.remove_spawned_gates()
-        sgate = self.spawn(ch, ax)
-        self._refresh_activation()
-
     @property
     def source_channels(self):
         """ Returns a set describing the source channels on which the gate is defined. """
@@ -408,7 +403,7 @@ class PlottableGate(AxesWidget):
             self._spawned_vertex_list.remove(event.info['caller'])
 
     def _update(self):
-        self.canvas.draw()
+        self.canvas.draw_idle()
 
     def remove(self):
         # IMPORTANT Do not remove spawned vertexes from the _list directly. Use 
@@ -441,13 +436,6 @@ class PlottableGate(AxesWidget):
     def vertex(self):
         return self._spawned_vertex_list
 
-    #def set_visible(self, visible):
-        #for artist in self.artist_list:
-            #artist.set_visible(visible)
-        #for vertex in to_list(self.vertex):
-            #vertex.set_visible(visible)
-        #self._update()
-
 class PolyGate(PlottableGate):
     def create_artist(self):
         self.poly = pl.Polygon(self.coordinates, color='k', fill=False)
@@ -560,7 +548,7 @@ class PolyDrawer(AxesWidget):
         self._update()
 
     def _update(self):
-        self.canvas.draw()
+        self.canvas.draw_idle()
 
     def _clean(self):
         self.disconnect_events()
@@ -613,6 +601,11 @@ class FCToolBar(object):
         self.set_active_gate(event.info['caller'])
 
     def create_gate_widget(self, kind):
+        def clean_drawing_tools():
+            self._drawing_tool.disconnect_events()
+            self.canvas.draw_idle()
+            self._drawing_tool = None
+
         def create_gate(*args):
             canceled = False # TODO allow drawing tool to cancel
             verts = args[0]
@@ -638,7 +631,7 @@ class FCToolBar(object):
                 gate.spawn(ch, self.ax)
                 self.add_gate(gate)
 
-            clean_drawing_tools(kind)
+            clean_drawing_tools()
 
         def start_drawing(kind):
             if kind == 'poly':
@@ -651,12 +644,11 @@ class FCToolBar(object):
                 self._drawing_tool = Cursor(self.ax, vertOn=1, horizOn=0)
 
             if isinstance(self._drawing_tool, Cursor):
-                self._drawing_tool.connect_event('button_press_event', lambda event : create_gate([(event.xdata, event.ydata)]))
+                def finish_drawing(event):
+                    self._drawing_tool.clear(None)
+                    return create_gate([(event.xdata, event.ydata)])
 
-        def clean_drawing_tools(kind):
-            self._drawing_tool.disconnect_events()
-            self._drawing_tool = None
-            self.canvas.draw()
+                self._drawing_tool.connect_event('button_press_event', finish_drawing)
 
         start_drawing(kind)
 
@@ -688,7 +680,6 @@ class FCToolBar(object):
         if self.sample is not None:
             self.current_channels = list(self.sample.channel_names[0:2])
             self.set_axis(self.current_channels, self.ax)
-            self.plot_data()
 
     def set_axis(self, channels, ax):
         """
@@ -698,10 +689,27 @@ class FCToolBar(object):
         """
         self.current_channels = channels
 
+        ## 
+        # Order of operations is important 
+        # because there is a clear command in 
+        # plot_data
+        # difficult to implement better b/c
+        # output from plotting functions from matplotlib
+        # don't return all required information.
+
+        # Remove existing gates
         for gate in self.gates:
-            gate.set_axis(channels, ax)
+            gate.remove_spawned_gates()
+        ## 
+        # Has a clear axis command inside!!
+        # which will "force kill" spawned gates
         self.plot_data()
 
+        for gate in self.gates:
+            sgate = gate.spawn(channels, ax)
+            gate._refresh_activation()
+
+
     def close(self):
         for gate in self.gates:
             gate.remove()
@@ -714,48 +722,33 @@ class FCToolBar(object):
     def plot_data(self):
         """ Plots the loaded data """
         if self.sample is None: return
-        sample = self.sample
-        ax = self.ax
 
-        if self._plt_data is not None:
-            if isinstance(self._plt_data, tuple):
-                # This is the case for histograms which return a tuple
-                patches = self._plt_data[2]
-                map(lambda x : x.remove(), patches)
-            else:
-                self._plt_data.remove()
-            del self._plt_data
-            self._plt_data = None
+        # Clear the axes
+        self.ax.cla()
 
+        ###
+        # Potential code to clear axis
+        # will be easier to use in future version
+        # of matplotlib
+        #
+        #    #if self._plt_data is not None:
+        #        ## HARD CODED FOR SPEED NEEDS FIXING
+        #        #if self._plt_data[0] == '1dhist':
+        #            #patches = self._plt_data[1]
+        #            #[p.remove() for p in patches] # removes patches
+        #        #elif self._plt_data[0] == '2dhist':
+        #            #axes_image = self._plt_data[1]
+        #            #print axes_image
+        #            #print type(axes_image)
+        #            #axes_image.remove()
+        #        #self._plt_data = None
 
         if self.current_channels is None:
-            self.current_channels = sample.channel_names[:2]
+            self.current_channels = self.sample.channel_names[:2]
 
         channels = self.current_channels
-
-        if len(channels) == 1: # Then histogram
-            self._plt_data = sample.plot(channels[0], ax=ax)
-            xlabel = self.current_channels[0]
-            ylabel = 'Counts'
-        else:
-            self._plt_data = sample.plot(channels, ax=ax)
-            xlabel = self.current_channels[0]
-            ylabel = self.current_channels[1]
-
-        if hasattr(self._plt_data, 'get_datalim'):
-            bbox = self._plt_data.get_datalim(self.ax.transData)
-            p0 = bbox.get_points()[0]
-            p1 = bbox.get_points()[1]
-
-            self.ax.set_xlim(p0[0], p1[0])
-            self.ax.set_ylim(p0[1], p1[1])
-        else:
-            # Then it's a histogram?
-            xlims = self._plt_data[1]
-            xlims = (xlims[0], xlims[-1])
-            self.ax.set_xlim(xlims)
-            self.ax.set_ylim(0, max(self._plt_data[0]))
-
+        channels_to_plot = channels[0] if len(channels) == 1 else channels
+        out = self.sample.plot(channels_to_plot, ax=self.ax)
         self.fig.canvas.draw()
 
     def get_generation_code(self):
@@ -797,11 +790,11 @@ def key_press_handler(event, canvas, toolbar=None):
     elif key in ['8']:
         print toolbar.get_generation_code()
 
-class Globals():
-    """ Used for testing only """
-    pass
-
 if __name__ == '__main__':
+    class Globals():
+        """ Used for testing only """
+        pass
+
     def example1():
         fig = figure()
         ax = fig.add_subplot(1, 4, 1)
@@ -861,6 +854,7 @@ if __name__ == '__main__':
         fig = pl.figure()
         ax = fig.add_subplot(1, 1, 1)
         manager = FCToolBar(ax)
+        #manager.load_fcs('../tests/data/FlowCytometers/FACSCaliburHTS/Sample_Well_A02.fcs')
         pl.show()
 
     example3()
