@@ -5,5 +5,5 @@ from IO.fcsreader import parse_fcs
 from core.containers import FCMeasurement, FCCollection, FCOrderedCollection, FCPlate
 from core.gates import ThresholdGate, IntervalGate, QuadGate, PolyGate
 import core.graph as graph
-from core.graph import plotFCM, plot_histogram2d
+from core.graph import plotFCM
 from core.util import build_FCplate_collection
